<?php namespace App\Http\Controllers;

use App\Libraries\ResponseCodes;
use App\models\Lead;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Validator;

class LeadsController extends Controller
{

    protected $_lead;
    protected $_validator;

//    use RESTActions;

    public function __construct(Lead $lead)
    {
        $this->_lead = $lead;
    }

    public function all()
    {
        return $this->respond(Response::HTTP_OK, $this->_lead->all());
    }

    public function index(Request $request)
    {

        $searchableColumns = array(
            'name' =>
                array(
                    'dataType' => 'like',
                    'join' => 'leads.name'
                ),
            'phone' =>
                array(
                    'dataType' => 'like',
                    'join' => 'leads.phone'
                ),
            'email' =>
                array(
                    'dataType' => 'like',
                    'join' => 'leads.email'
                ),
        );
        $perpage = (!empty($request->get('per-page'))) ? $request->get('per-page') : 30;

        $model = Lead::search($request, $searchableColumns)->orderBy("id","DESC")->paginate($perpage);

        $data = $model->toArray();

        if (!empty($data['data'])) {
            $this->_response['data'] = ["leads" => $data['data']];
        } else {
            $this->_response['code'] = ResponseCodes::DATA_NOT_FOUND;
            $this->_response['message'] = ResponseCodes::DATA_NOT_FOUND_MSG;
        }

        return $this->response();
    }

    public function get($id)
    {
        $model = $this->_lead->find($id);
        if (is_null($model)) {
            return $this->respond(Response::HTTP_NOT_FOUND);
        }
        return $this->respond(Response::HTTP_OK, $model);
    }

    public function add(Request $request)
    {
        $this->validate($request, $this->_lead->rules);
        return $this->respond(Response::HTTP_CREATED, $this->_lead->create($request->all()));
    }

    public function put(Request $request, $id)
    {
        $this->validate($request, $this->_lead->rules);
        $model = $this->_lead->find($id);
        if (is_null($model)) {
            return $this->respond(Response::HTTP_NOT_FOUND);
        }
        $model->update($request->all());

        return $this->respond(Response::HTTP_OK, $model);
    }

    public function remove($id)
    {
        if (is_null($this->_lead->find($id))) {
            return $this->respond(Response::HTTP_NOT_FOUND);
        }
        $this->_lead->destroy($id);
        return $this->respond(Response::HTTP_NO_CONTENT);
    }

    protected function respond($status, $data = [])
    {
        return response()->json($data, $status);
    }

    /**
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     * @throws \Illuminate\Validation\ValidationException
     */
    public function store(Request $request): \Illuminate\Http\JsonResponse
    {
        try {
            $this->_validator = Validator::make($request->all(), $this->_lead->rules);
            if ($this->_validator->fails())
                throw new \Exception("Invalid Data");

            $this->_lead = $this->_lead->create($request->all());

        } catch (\Exception $exception) {
            $errors['error_fields'] = $this->_validator->errors();
            return $this->respond(Response::HTTP_CREATED, ["status" => 0, "data" => $errors, "msg" => $exception->getMessage()]);
        }

        return $this->respond(Response::HTTP_CREATED, ["status" => 1, "data" => $this->_lead]);
    }

    /**
     * @param Request $request
     * @param $errors
     * @return \Illuminate\Http\JsonResponse
     */
    public function freeStore(Request $request): \Illuminate\Http\JsonResponse
    {
        try {

            if (empty($request->get('phone')) && empty($request->get('email')))
                $this->_validator = Validator::make(['phone' => $request->get("phone")], ["phone" => "required|numeric|min:11"]);

            if (!empty($request->get('phone')))
                $this->_validator = Validator::make(['phone' => $request->get("phone")], ["phone" => "required|numeric|min:11"]);

            if (!empty($request->get('email')))
                $this->_validator = Validator::make(['email' => $request->get("email")], ["email" => "required|email"]);

            if ($this->_validator->fails())
                throw new \Exception("Invalid Data");

            $this->_validator = Validator::make($request->all(), $this->_lead->rules);
            if ($this->_validator->fails())
                throw new \Exception("Invalid Data");

            if ($request->get('id'))
                $this->_lead = $this->_lead->findOrFail($request->get('id'));

            $this->_lead->loadModel($request->all());

//            if (empty($this->_lead->getDirty()))
//                throw new \Exception("Already exist the same data, nothing changed.");

            $this->_lead->saveOrFail($request->all());

        } catch
        (\Exception $exception) {
            $errors['error_fields'] = $this->_validator->errors();
            return $this->respond(Response::HTTP_CREATED, ["status" => 0, "data" => $errors, "msg" => $exception->getMessage()]);
        }

        return $this->respond(Response::HTTP_CREATED, ["status" => 1, "data" => $this->_lead]);
    }

    /**
     * @param array $input
     * @param string $validate
     * @return array
     */
    protected function singleValueValidate(array $input, string $validate): array
    {
        $validation = [];
        foreach ($input as $key => $item):
            $validation[$key] = $validate;
        endforeach;

        if (!empty($validation))
            $validator = Validator::make($input, $validation);

        return $validator;
    }


    protected function response()
    {
        return response()->json($this->_response, 200, [], JSON_UNESCAPED_SLASHES | JSON_PRETTY_PRINT);
    }
}
