<?php


namespace App\Http\Controllers;


use App\Helpers\DateTimeHelper;

use App\models\Activity;
use App\models\Cart;
use App\models\Disposition;
use App\models\Lead;
use App\models\MealPlan;
use App\models\MealRecipe;
use App\models\Member;
use App\models\MemberAddresses;
use App\models\MemberMealPlan;
use App\models\MemberReservation;
use App\models\Order;
use App\models\Param;
use App\models\Rating;
use App\models\Reservation;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class MemberController extends Controller
{
    const MEAL_CARD_ID = 3;
    const PACKAGES_CARD = 2;
    const ACTIVE_PACKAGE_CARD = 1;
    const CONTACT_ALERT = 4;
    const WEIGHT_CARD = 5;
    const NOTIFICATION = 6;
    const SHOP_BANNER = 7;


    /**
     * Api`s list
     *
     *
     *
     * - Home dashboard
     * - reservations listing -- calendar --param meal_plan_id
     * - Package order listing
     * - Package order detail -- param member reservation id
     * - invoice listing --pagination
     * - address listing
     * - add address
     * - change address
     * - allergy update / listing
     * - update profile --with update weight / height dispo
     * - add cart --add cart on order
     * - package cart
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse
     */

    public function update(Request $request)
    {
        try {
            DB::beginTransaction();
            // Validate the input and return correct response
            $member = $request->auth;
            $activities = (!empty($request->get("activities"))) ? $request->get("activities") : null;

            foreach ($request->except("phone") as $attribute => $value) {
                if ($member->existAttribute($attribute)) {
                    if (!empty($value)) {
                        $member->setAttribute($attribute, $value);
                    }
                }
            }

            $member->save();

            if ($request->has("address"))
                $member->addPrimaryAddress($request->get("address"));

            if ($activities) {
                foreach ($activities as $activityId):
                    $activity = Activity::findOrFail($activityId['id']);
                    if (!empty($activity))
                        $member->activities()->syncWithoutDetaching($activity);
                endforeach;
            }
            DB::commit();
            $member = Member::where("id", $member->id)->first();
            $token = $this->payLoad($member);
            return $this->sendResult("Profile Updated", 200, ["member" => $member->toArray(), "token" => $token]);
        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));
        }
    }

    public function home(Request $request)
    {
        //initials ..
        $date = DateTimeHelper::getLocalNow();
        $dateStart = $date->copy()->subDay(1)->format("Y-m-d");
        $member = $request->auth;
        $scale = $member->memberScale;
        $mealplanId = MealPlan::ASSASSINATE_WEIGHT_ID;
        $isExpire = false;
        $data = [];

        $data[] = [
            "title" => "Visit Shop",
            "type" => self::SHOP_BANNER,
            "data" => ["msg" => "- Delivery (available only in Clifton, Defence and PECSH) \n- TakeAway \n- Dining"]
        ];

//        $param = Param::where("key", "DashboardNote")->first();
//        if ($param) {
//            $data[] = [
//                "title" => "Notification note",
//                "type" => self::NOTIFICATION,
//                "data" => ["msg" => "sadfsa asd fas df asd"]
//            ];
//        }

        $lastActivePlan = DB::select("
                        SELECT
                            member_meal_plans.id,
                            member_meal_plans.meal_plan_id,
                            member_meal_plans.startOn,
                            member_meal_plans.expireOn,
                            meal_plans.title as mealplan,
                            COUNT(member_reservations.id) AS totalDays,
                            DATEDIFF(member_meal_plans.expireOn,member_meal_plans.startOn) AS planTotalDays,
                            COUNT(IF(member_reservations.status IN ('" . MemberReservation::STATUS_CANCELED . "'),1,NULL)) 'canceled',
                            COUNT(IF(member_reservations.status NOT IN ('" . MemberReservation::STATUS_CANCELED . "'),1,NULL)) 'delivered'
                        FROM
                            member_meal_plans
                            INNER JOIN
                            meal_plans ON meal_plans.id = member_meal_plans.meal_plan_id
                                LEFT JOIN
                            member_reservations ON member_meal_plans.id = member_reservations.member_meal_plan_id

                            where member_meal_plans.member_id = {$member->id}
                             group by member_meal_plans.id
                            ORDER  by member_meal_plans.expireOn desc limit 1");


        $data[] = [
            "title" => "Member",
            "type" => self::WEIGHT_CARD,
            "data" => ["member" => $member->toArray(), "scale" => (!empty($scale)) ? $scale->toArray() : null]
        ];


        if (!empty($lastActivePlan)) {
            $lastActivePlan = $lastActivePlan[0];

            $mealplanId = $lastActivePlan->meal_plan_id;

            if (Carbon::parse($lastActivePlan->expireOn) <= $date) {
                $isExpire = true;
                $data[] = [
                    "title" => "Alert",
                    "type" => self::CONTACT_ALERT,
                    "data" => [
                        "title" => "Package Expired",
                        "btnTitle" => "Call for Renew Package",
                        "request" => "Renew My Package",
                        "msg" => "Hello {$member->name}, Your enrollment duration of {$lastActivePlan->mealplan} has been Expired on {$lastActivePlan->expireOn}.",
                        "phone" => "+923322223429"]
                ];
            }

            $data[] = [
                "title" => "Active Package",
                "type" => self::ACTIVE_PACKAGE_CARD,
                "data" => [
                    "packageDetail" => [
                        "totalDays" => $lastActivePlan->totalDays ?? 0,
                        "planTotalDays" => $lastActivePlan->planTotalDays ?? 0,
                        "remainingDays" => $lastActivePlan->planTotalDays - $lastActivePlan->totalDays,
                        "canceled" => $lastActivePlan->canceled,
                        "delivered" => $lastActivePlan->delivered,
                        "mealplan" => $lastActivePlan->mealplan,
                        "expireOn" => $lastActivePlan->expireOn,
                        "startOn" => $lastActivePlan->startOn,
                    ]
                ]
            ];

        }

        if ($isExpire || empty($lastActivePlan)) {
            $data[] = [
                "title" => "Meal Plans",
                "type" => self::PACKAGES_CARD,
                "data" => [
                    "mealplans" => MealPlan::where("active", 1)->get()->toArray()
                ]
            ];
        }

        $mealplans = MealPlan::join("member_meal_plans", "meal_plans.id", "=", "member_meal_plans.meal_plan_id")->where("member_meal_plans.member_id", $member->id)->select("meal_plans.*")->get();

        if ($mealplans) {
            $data[] = [
                "title" => "Previous Package",
                "type" => self::PACKAGES_CARD,
                "data" => ["mealplans" => $mealplans->toArray()
                ]
            ];
        }

        $date = $date->format("Y-m-d");
        $reservations = $this->getReservationWithDate($member->id, $mealplanId, $dateStart, $date);
        foreach ($reservations as $item) {
            if ($date == $item->onDate)
                $data[] = [
                    "title" => "Today Meals",
                    "type" => self::MEAL_CARD_ID,
                    "data" => [
                        "reservation" => $item->toArray()
                    ]
                ];
            if ($dateStart == $item->onDate)
                $data[] = [
                    "title" => "Yesterday Meals",
                    "type" => self::MEAL_CARD_ID,
                    "data" => [
                        "reservation" => $item->toArray()
                    ]
                ];
        }

        return $this->sendResult("Welcome {$member->name}", 200, [
                "attributes" => $data
            ]
        );
    }

    public function calendar(Request $request)
    {
        $date = DateTimeHelper::getNow();
        $member = $request->auth;

        $activeMealplanQuery = $request->auth->activeMealplan()->where("member_meal_plans.expireOn", ">=", $date)->first();

        $dateEnd = $date->copy()->addDay(15)->format("Y-m-d");
        $dateStart = $date->copy()->subDay(15)->format("Y-m-d");

        $mealplanId = null;

        if ($request->has("meal_plan_id")) {
            $mealplanId = $request->get("meal_plan_id");
        }

        $activeMealplan = null;
        if ($activeMealplanQuery) {
            if (!$mealplanId)
                $mealplanId = $activeMealplanQuery->meal_plan_id;

            if ($mealplanId == $activeMealplanQuery->meal_plan_id) {
                $activeMealplan = $activeMealplanQuery;
                $dateEnd = $activeMealplanQuery->expireOn;
                $dateStart = $activeMealplanQuery->startOn;
            }
        }
        if (!$mealplanId)
            $mealplanId = MealPlan::ASSASSINATE_WEIGHT_ID;

        $query = $this->getReservationWithDate($member->id, $mealplanId, $dateStart, $dateEnd);

        $mealplans = MealPlan::where('active', 1)->select("meal_plans.*", DB::raw("CONCAT('" . env('IMAGE_DIRECTORY') . "/','',meal_plans.logo) AS logo"))->get()->toArray();

        return $this->sendResult("Reservations", 200, [
                "activeMealPlan" => ($activeMealplan) ? $activeMealplan->toArray() : null,
                "meal_plans" => $mealplans,
                'reservations' => ($query) ? $query->toArray() : null]
        );
    }

    public function memberReservations(Request $request)
    {
        //reservations plan
        $query = MemberReservation::join("reservations", 'reservations.id', '=', 'member_reservations.reservation_id');

        $query->leftJoin('meals as breakfastMeal', 'breakfastMeal.id', '=', 'reservations.breakfast_id');
        $query->leftJoin('files as breakfastFile', 'breakfastFile.id', '=', 'breakfastMeal.cover_id');

        $query->leftJoin('meals as lunchMeal', 'lunchMeal.id', '=', 'reservations.lunch_id');
        $query->leftJoin('files as lunchFile', 'lunchFile.id', '=', 'lunchMeal.cover_id');

        $query->leftJoin('meals as snackMeal', 'snackMeal.id', '=', 'reservations.snack_id');
        $query->leftJoin('files as snackFile', 'snackFile.id', '=', 'snackMeal.cover_id');

        $query->leftJoin('meals as dinnerMeal', 'dinnerMeal.id', '=', 'reservations.dinner_id');
        $query->leftJoin('files as dinnerFile', 'dinnerFile.id', '=', 'dinnerMeal.cover_id');

        $query->leftJoin('meal_plans', 'meal_plans.id', '=', 'reservations.meal_plan_id');
        $query->leftJoin('users', 'users.id', '=', 'member_reservations.user_id');

        $imagePath = env("IMAGE_BASE_URL");
        //select columns ...
        $query->select(
            'member_reservations.*',
            "reservations.id as reservationId",
            "reservations.breakfast_id",
            "reservations.lunch_id",
            "reservations.snack_id",
            "reservations.dinner_id",
            "reservations.meal_plan_id",

            "reservations.onDate",

            'breakfastMeal.title as breakfast',
            'breakfastMeal.totalCalories as breakfastCalories',
            'breakfastMeal.totalGrams as breakfastGrams',
            'breakfastMeal.rating as breakfastRating',
            DB::raw("CONCAT('$imagePath/',breakfastFile.filePath,'/',breakfastFile.title) as breakfastLogo"),

            'lunchMeal.title as lunch',
            'lunchMeal.totalCalories as lunchCalories',
            'lunchMeal.totalGrams as lunchGrams',
            'lunchMeal.rating as lunchRating',
            DB::raw("CONCAT('$imagePath/',lunchFile.filePath,'/',lunchFile.title) as lunchLogo"),

            'snackMeal.title as snack',
            'snackMeal.totalCalories as snackCalories',
            'snackMeal.totalGrams as snackGrams',
            'snackMeal.rating as snackRating',
            DB::raw("CONCAT('$imagePath/',snackFile.filePath,'/',snackFile.title) as snackLogo"),

            'dinnerMeal.title as dinner',
            'dinnerMeal.totalCalories as dinnerCalories',
            'dinnerMeal.totalGrams as dinnerGrams',
            'dinnerMeal.rating as dinnerRating',
            DB::raw("CONCAT('$imagePath/',dinnerFile.filePath,'/',dinnerFile.title) as dinnerLogo"),

            'member_reservations.id as memberReservationId',
            'meal_plans.title as meal_plan',
            'users.name as riderName'
        );

        $query->where(["member_reservations.member_id" => $request->auth->id]);
        $query->groupBy("member_reservations.id");
        $query->orderBy("reservations.onDate", "desc");

        $query = $query->paginate($request->get("per-page", 10))->toArray();

        return $this->sendResult("Member orders", 200, ['member_reservations' => $query["data"]]);
    }

    public function memberReservation(Request $request, $id)
    {
        //reservations plan
        $query = MemberReservation::join("reservations", 'reservations.id', '=', 'member_reservations.reservation_id');

        $query->leftJoin('meals as breakfastMeal', 'breakfastMeal.id', '=', 'reservations.breakfast_id');
        $query->leftJoin('files as breakfastFile', 'breakfastFile.id', '=', 'breakfastMeal.cover_id');

        $query->leftJoin('meals as lunchMeal', 'lunchMeal.id', '=', 'reservations.lunch_id');
        $query->leftJoin('files as lunchFile', 'lunchFile.id', '=', 'lunchMeal.cover_id');

        $query->leftJoin('meals as snackMeal', 'snackMeal.id', '=', 'reservations.snack_id');
        $query->leftJoin('files as snackFile', 'snackFile.id', '=', 'snackMeal.cover_id');

        $query->leftJoin('meals as dinnerMeal', 'dinnerMeal.id', '=', 'reservations.dinner_id');
        $query->leftJoin('files as dinnerFile', 'dinnerFile.id', '=', 'dinnerMeal.cover_id');

        $query->leftJoin('meal_plans', 'meal_plans.id', '=', 'reservations.meal_plan_id');
        $query->leftJoin('users', 'users.id', '=', 'member_reservations.user_id');

        $imagePath = env("IMAGE_BASE_URL");
        //select columns ...
        $query->select(
            'member_reservations.*',

            "reservations.breakfast_id",
            "reservations.lunch_id",
            "reservations.snack_id",
            "reservations.dinner_id",


            'meal_plans.title as meal_plan',
            'users.name as riderName',

            "reservations.onDate",
            "reservations.id as reservationId",

            'breakfastMeal.title as breakfast',
            'breakfastMeal.totalCalories as breakfastCalories',
            'breakfastMeal.totalGrams as breakfastGrams',
            'breakfastMeal.rating as breakfastRating',
            DB::raw("CONCAT('$imagePath/',breakfastFile.filePath,'/',breakfastFile.title) as breakfastLogo"),

            'lunchMeal.title as lunch',
            'lunchMeal.totalCalories as lunchCalories',
            'lunchMeal.totalGrams as lunchGrams',
            'lunchMeal.rating as lunchRating',
            DB::raw("CONCAT('$imagePath/',lunchFile.filePath,'/',lunchFile.title) as lunchLogo"),

            'snackMeal.title as snack',
            'snackMeal.totalCalories as snackCalories',
            'snackMeal.totalGrams as snackGrams',
            'snackMeal.rating as snackRating',
            DB::raw("CONCAT('$imagePath/',snackFile.filePath,'/',snackFile.title) as snackLogo"),

            'dinnerMeal.title as dinner',
            'dinnerMeal.totalCalories as dinnerCalories',
            'dinnerMeal.totalGrams as dinnerGrams',
            'dinnerMeal.rating as dinnerRating',
            DB::raw("CONCAT('$imagePath/',dinnerFile.filePath,'/',dinnerFile.title) as dinnerLogo")

        );

        $query->where(["member_reservations.id" => $id, "member_reservations.member_id" => $request->auth->id]);
        $query = $query->groupBy("member_reservations.id");
        $memberReservation = $query->first();
        $mealIds = $nutrition = [];
        if ($memberReservation->breakfast_id)
            $mealIds[] = $memberReservation->breakfast_id;
        if ($memberReservation->lunch_id)
            $mealIds[] = $memberReservation->lunch_id;
        if ($memberReservation->snack_id)
            $mealIds[] = $memberReservation->snack_id;
        if ($memberReservation->dinner_id)
            $mealIds[] = $memberReservation->dinner_id;

        if ($mealIds) {
            $newQuery = MealRecipe::query();
            $nutrition = $newQuery->join('recipe_ingredients', 'meal_recipes.recipe_id', '=', 'recipe_ingredients.recipe_id')
                ->join('ingredient_nutrition', 'recipe_ingredients.ingredient_id', '=', 'ingredient_nutrition.ingredient_id')
                ->join('nutrition', 'nutrition.id', '=', 'ingredient_nutrition.nutrition_id')
                ->select('nutrition.title', 'nutrition.unit', DB::raw('SUM( recipe_ingredients.grams * ingredient_nutrition.value) as `value`'))
                ->whereIn('meal_recipes.meal_id', $mealIds)
                ->groupBy('nutrition.id')->get()->toArray();
        }
        $dispositions = Disposition::leftJoin("reasons", "reasons.id", "=", "dispositions.reason_id")
            ->select("dispositions.status", "dispositions.comment", "dispositions.created_at", "reasons.title as reason")
            ->where(['object' => Disposition::OBJECT_MEMBER_RESERVATIONS, "objectId" => $id])
            ->orderBy("dispositions.id", "desc")->get()->toArray();

        return $this->sendResult("Member Reservations", 200, ['member_reservation' => $memberReservation->toArray(), "dispositions" => $dispositions, "nutrition" => $nutrition]);
    }

    public
    function mealCancelPost(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "onDate" => "required",
            "meal_plan_id" => "required",
            "reason_id" => "required",
        ]);
        try {
            DB::beginTransaction();
            // Validate the input and return correct response
            if ($validator->fails())
                throw new \Exception($validator->getMessageBag()->first());

            $member = $request->auth;
            $onDate = $request->get("onDate");
            $meal_plane_id = $request->get("meal_plan_id");
            //check if valid ...
            $memberMealPlanId = $member->isValidMealplanOfMember($meal_plane_id, $onDate);

            if ($memberMealPlanId == 0)
                throw new \Exception("Member is not having reservation on this date " . $onDate);

            $reservation = Reservation::where(["onDate" => $onDate, "meal_plan_id" => $meal_plane_id])->first();

            if (empty($reservation))
                throw new \Exception("No Reservation is Found ...");

            $memberReservation = MemberReservation::join("reservations", "reservations.id", "=", "member_reservations.reservation_id")
                ->where("reservations.onDate", $onDate)
                ->where("member_reservations.member_id", $member->id)
                ->select("member_reservations.*")
                ->first();

            if ($memberReservation) {
                if ($memberReservation->status == MemberReservation::STATUS_COOKING) {
                    throw new \Exception("Reservation is on Cooking Process");
                }
                if ($memberReservation->status == MemberReservation::STATUS_CANCELED) {
                    throw new \Exception("{$onDate} reservation is already canceled");
                }
                throw new \Exception("The Reservation is already in {$memberReservation->status}");
            }

            $member_reservation = new MemberReservation([
                    "reservation_id" => $reservation->id,
                    "member_id" => $member->id,
                    "member_meal_plan_id" => $memberMealPlanId,
                    "status" => MemberReservation::STATUS_CANCELED,
                    "type" => MemberReservation::TYPE_PACKAGE,
                ]
            );

            //breakfast
            $member_reservation->is_breakfast = 0;
            //lunch
            $member_reservation->is_lunch = 0;
            //dinner
            $member_reservation->is_dinner = 0;
            //snack
            $member_reservation->is_snack = 0;

            if (!$member_reservation->save())
                throw new \Exception("An error occur while canceling reservation");

            $member_reservation->addDisposition($request->get("comment", "{$member->name} {$onDate} Day Reservation {$member_reservation->status}"), $request->get("reason_id"));

            $memberMealPlan = MemberMealPlan::findOrFail($memberMealPlanId);
            $oldExpireDate = Carbon::parse($memberMealPlan->expireOn);
            $memberMealPlan->expireOn = $oldExpireDate->copy()->addDay(1);

            if (!$memberMealPlan->save())
                throw new \Exception("An error occur while saving MemberMealPlan expireOn Date");

            $memberMealPlan->addDisposition("{$onDate} Reservation Canceled by member {$member->name}");
            DB::commit();
            return $this->sendResult("{$onDate} Reservations Status has been Canceled", 200, []);
        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));
        }
    }

    public
    function addresses(Request $request)
    {
        $member = Member::where("id", $request->auth->id)->with("primaryAddress", "secondaryAddress")->first();

        return $this->sendResult("My Addresses", 200, ['member' => $member->toArray()]);
    }

    public
    function editAddress(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "type" => "required"
        ]);
        try {
            DB::beginTransaction();
            // Validate the input and return correct response
            if ($validator->fails())
                throw new \Exception($validator->getMessageBag()->first());

            $member = $request->auth;
            $type = $request->get("type");
            if ($type == MemberAddresses::TYPE_PRIMARY) {
                if (!empty($request->get("address"))) {
                    $member->addPrimaryAddress($request->get("address"), $request->get("address_note"));
                }
            }
            if ($type == MemberAddresses::TYPE_SECONDARY) {

                $data = [];
                $data['address_note'] = $request->get("address_note", null);
                if (!empty($request->get("days"))) {
                    $days = explode("-", $request->get("days"));
                    foreach ($days as $day => $val) {
                        $data[$val] = 1;
                    }

                }
                $member->addSecondaryAddress($request->get("address"), $data);

            }
            DB::commit();
            $member = Member::where("id", $request->auth->id)->with("primaryAddress", "secondaryAddress")->first();

            return $this->sendResult("Address Has Been Saved", 200, ["member" => $member->toArray()]);
        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));
        }
    }


    public
    function addCart(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "type" => "required",
            "onDate" => "required",
            "meal_plan_id" => "required",
        ]);
        try {
            DB::beginTransaction();
            // Validate the input and return correct response
            if ($validator->fails())
                throw new \Exception($validator->getMessageBag()->first());

            $mealplan = MealPlan::findOrFail($request->get("meal_plan_id"));
            $member = $request->auth;
            $lead = new Lead([
                    "name" => $member->name,
                    "phone" => $member->phone,
                    "email" => $member->email,
                    "type" => "MOBILE",
                ]
            );
            $lead->save();
            $lead->addDisposition("Cart Entry through mobile");
            $cartData = [
                "type" => $request->get("type"),
                "meal_plan_id" => $request->get("meal_plan_id"),
                "mealplan" => $mealplan->title,
                "startDate" => $request->get("onDate"),
                "packageAmount" => $mealplan->amount,
                "perDayAmount" => $mealplan->perDayAmount,
            ];
            $cart = new Cart([
                'lead_id' => $lead->id,
                "cartData" => json_encode($cartData),
                'isMobile' => 1,
            ]);

            $cart->save();

            DB::commit();
            return $this->sendResult("Your Order has been added, Our Representative will call you", 200);
        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));
        }
    }


    public function reviewMeal(Request $request)
    {
        $validator = Validator::make($request->all(), [
            "mealId" => "required",
            "rating" => "required",
        ]);
        try {
            DB::beginTransaction();
            // Validate the input and return correct response
            if ($validator->fails())
                throw new \Exception($validator->getMessageBag()->first());

            $member = $request->auth;

            $rating = new Rating([
                "member_id" => $member->id,
                "rate" => $request->get('rating'),
                "object" => "MEALS",
                "objectId" => $request->get('mealId'),
                "reason_id" => $request->get('reasonId'),
                "comment" => $request->get('comment'),
            ]);

            $rating->save();

            DB::commit();
            return $this->sendResult("Thank you", 200);

        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));
        }
    }


    public function shopOrders(Request $request)
    {
        //orders plan
        $query = Order::join("shops", 'orders.shop_id', '=', 'shops.id');
        $query->join('order_items', 'orders.id', '=', 'order_items.order_id');
        $query->join('meals', 'meals.id', '=', 'order_items.meal_id');
        $query->select(
            'orders.*',
            'shops.title as shopTitle',
            'shops.phone as shopPhone',
            DB::raw('group_concat(meals.title) as title'),
            DB::raw('sum(meals.totalGrams) as totalGrams'),
            DB::raw('sum(meals.totalCalories) as totalCalories')
        );
        $query->where(["orders.member_id" => $request->auth->id]);
        $query->groupBy("orders.id");
        $query->orderBy("orders.id", "desc");

        $query = $query->paginate($request->get("per-page", 10))->toArray();

        return $this->sendResult("Shop orders", 200, ['orders' => $query["data"]]);
    }

    public function shopOrder($id)
    {
        $order = Order::where("id", $id)->with('shop', 'orderItems')->first();
        if (empty($order))
            return $this->sendResult("No Order Found", config('codes.DATA_NOT_FOUND'));
        $order = $order->toArray();
        $mealIds = [];
        $order['nutrition'] = null;
        foreach ($order['order_items'] as $item) {
            $mealIds[] = $item['meal_id'];
        }
        if (!empty($mealIds)) {
            $newQuery = MealRecipe::query();
            $nutrition = $newQuery->join('recipe_ingredients', 'meal_recipes.recipe_id', '=', 'recipe_ingredients.recipe_id')
                ->join('ingredient_nutrition', 'recipe_ingredients.ingredient_id', '=', 'ingredient_nutrition.ingredient_id')
                ->join('nutrition', 'nutrition.id', '=', 'ingredient_nutrition.nutrition_id')
                ->select('nutrition.title', 'nutrition.unit', DB::raw('SUM( recipe_ingredients.grams * ingredient_nutrition.value) as `value`'))
                ->whereIn('meal_recipes.meal_id', $mealIds)
                ->groupBy('nutrition.id')->get()->toArray();
            $order['nutrition'] = $nutrition;
        }

        return $this->sendResult("Order", 200, ['order' => $order]);
    }

    /**
     * @param $member
     * @param int $mealplanId
     * @param static $date
     * @param $dateEnd
     * @return mixed
     */
    private
    function getReservationWithDate($memberId, $mealplanId, $dateStart, $dateEnd)
    {
        $query = Reservation::leftJoin('meals as breakfastMeal', 'breakfastMeal.id', '=', 'reservations.breakfast_id');
        $query->leftJoin('files as breakfastFile', 'breakfastFile.id', '=', 'breakfastMeal.cover_id');

        $query->leftJoin('meals as lunchMeal', 'lunchMeal.id', '=', 'reservations.lunch_id');
        $query->leftJoin('files as lunchFile', 'lunchFile.id', '=', 'lunchMeal.cover_id');

        $query->leftJoin('meals as snackMeal', 'snackMeal.id', '=', 'reservations.snack_id');
        $query->leftJoin('files as snackFile', 'snackFile.id', '=', 'snackMeal.cover_id');

        $query->leftJoin('meals as dinnerMeal', 'dinnerMeal.id', '=', 'reservations.dinner_id');
        $query->leftJoin('files as dinnerFile', 'dinnerFile.id', '=', 'dinnerMeal.cover_id');

        $query->leftJoin('meal_plans', 'meal_plans.id', '=', 'reservations.meal_plan_id');
        $query->leftJoin('member_reservations', function ($join) use ($memberId) {
            $join->on('reservations.id', '=', 'member_reservations.reservation_id');
            $join->on('member_reservations.member_id', DB::raw($memberId));
        });

        $imagePath = env("IMAGE_BASE_URL");
        //select columns ...
        $query->select(

            "reservations.id as reservationId",
            "reservations.breakfast_id",
            "reservations.lunch_id",
            "reservations.snack_id",
            "reservations.dinner_id",
            "reservations.meal_plan_id",
            "reservations.onDate",

            'breakfastMeal.title as breakfast',
            'breakfastMeal.totalCalories as breakfastCalories',
            'breakfastMeal.totalGrams as breakfastGrams',
            'breakfastMeal.rating as breakfastRating',
            'member_reservations.is_breakfast',
            DB::raw("CONCAT('$imagePath/',breakfastFile.filePath,'/',breakfastFile.title) as breakfastLogo"),

            'lunchMeal.title as lunch',
            'lunchMeal.totalCalories as lunchCalories',
            'lunchMeal.totalGrams as lunchGrams',
            'lunchMeal.rating as lunchRating',
            'member_reservations.is_lunch',
            DB::raw("CONCAT('$imagePath/',lunchFile.filePath,'/',lunchFile.title) as lunchLogo"),

            'snackMeal.title as snack',
            'snackMeal.totalCalories as snackCalories',
            'snackMeal.totalGrams as snackGrams',
            'snackMeal.rating as snackRating',
            'member_reservations.is_snack',
            DB::raw("CONCAT('$imagePath/',snackFile.filePath,'/',snackFile.title) as snackLogo"),

            'dinnerMeal.title as dinner',
            'dinnerMeal.totalCalories as dinnerCalories',
            'dinnerMeal.totalGrams as dinnerGrams',
            'dinnerMeal.rating as dinnerRating',
            'member_reservations.is_dinner',
            DB::raw("CONCAT('$imagePath/',dinnerFile.filePath,'/',dinnerFile.title) as dinnerLogo"),

            'member_reservations.status',
            'member_reservations.id as memberReservationId',
            'meal_plans.title as meal_plan'
        );

        $query->where(["reservations.meal_plan_id" => $mealplanId]);
        $query->whereBetween("reservations.onDate", array($dateStart, $dateEnd));

        $query->groupBy("reservations.id", "member_reservations.id");
        $query->orderBy("reservations.onDate", "DESC");

        return $query->get();
    }


}
