<?php namespace App\Http\Controllers;

use App\Helpers\DateTimeHelper;
use App\models\Activity;
use App\models\Lead;
use App\models\Meal;
use App\models\MealPlan;
use App\models\Coupon;

use App\models\Order;
use App\models\OrderItem;
use App\models\Param;
use App\models\Shop;
use Illuminate\Http\Request;

use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;

class PublicController extends Controller
{
    /**
     * API To Be Made
     *
     * - force update ..
     * - packages
     * - reasons
     * - shop listing
     * - shop detail
     * -meal details
     *
     **/

    public function forceUpdate(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'device_type' => 'required',
            'version' => 'required'
        ]);

        if ($validator->fails()) {
            return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), $validator->errors(), $request);
        }

        $message = '';
        $params = Param::where('key', $request->get('device_type'))->first();
        $version_int = 0;
        if ($params) {
            $db_version = $params['value'];
            $user_version = $request->get('version');

            if ($db_version == $user_version) {
                $version_int = 0;
                $message = 'You have already on updated version';
            } elseif ($db_version != $user_version && $db_version - floor($user_version) < 1) {
                $version_int = 1;
                $message = "we've upgraded Dietbox please update your app and enjoy added functionality";
            } elseif ($db_version != $user_version && $db_version - floor($user_version) >= 1) {
                $version_int = 2;
                $message = "we've upgraded dietbox please update your app and enjoy added functionality";
            }
        }

        return $this->sendResult($message, config('codes.SUCCESS'), ['version_int' => $version_int], $request);
    }

    public function packages()
    {
        $mealplans = MealPlan::where('active', 1)->select("meal_plans.*", DB::raw("CONCAT('" . env('IMAGE_DIRECTORY') . "/','',meal_plans.logo) AS logo"))->get()->toArray();

        return $this->sendResult("Meal Plans", 200, ['mealplans' => $mealplans]);
    }

    public function reasons(Request $request)
    {
        $parent = $request->get("parent");
        $child = $request->get("child");
        $query = DB::table("reasons");
        if ($parent)
            $query->where("parent", $parent);
        if ($child)
            $query->where("child", $child);
        $data = $query->get();
        return $this->sendResult("Reasons", 200, ['reasons' => $data]);
    }

    public function activities(Request $request)
    {

        $data = Activity::get()->toArray();
        return $this->sendResult("Activities", 200, ['activities' => $data]);
    }

    public function shops()
    {
        $shops = Shop::where('active', 1)->select("shops.*", DB::raw("CONCAT('" . env('IMAGE_DIRECTORY') . "/',files.filePath,'/',files.title) AS logo"))
            ->leftJoin("files", "files.id", "=", "shops.cover_id")
            ->get()->toArray();

        return $this->sendResult("Shops", 200, ['shops' => $shops]);
    }

    public function getShop($id)
    {
        $shops = Shop::where('shops.id', $id)
            ->select("shops.*", DB::raw("CONCAT('" . env('IMAGE_DIRECTORY') . "/',files.filePath,'/',files.title) AS logo"))
            ->leftJoin("files", "files.id", "=", "shops.cover_id")
            ->first()->toArray();

        return $this->sendResult("Shops", 200, ['shops' => $shops]);
    }

    public function meal(Request $request, $id)
    {
        $meal = Meal::leftJoin("files", "files.id", "=", "meals.cover_id")
            ->select(
                "meals.id",
                "meals.title",
                "meals.description",
                "meals.rating",
                "meals.totalGrams",
                "meals.totalCalories",
                "meals.forShop",
                "meals.type",
                "meals.perOrderAmount",
                DB::raw("CONCAT('" . env('IMAGE_BASE_URL') . "/',files.filePath,'/',files.title) AS logo"))
            ->where(['meals.id' => $id])
            ->first();
        $meal->setAppends(['recipes', 'nutrition']);
        return $this->sendResult("{$meal->title}", 200, ['meal' => $meal]);
    }

    public function shopMeals(Request $request)
    {
        $meal = Meal::leftJoin("files", "files.id", "=", "meals.cover_id")
            ->select(
                "meals.id",
                "meals.title",
                "meals.description",
                "meals.rating",
                "meals.totalGrams",
                "meals.totalCalories",
                "meals.forShop",
                "meals.type",
                "meals.perOrderAmount",
                DB::raw("CONCAT('" . env('IMAGE_BASE_URL') . "/',files.filePath,'/',files.title) AS logo"))
            ->where(['forShop' => 1])
            ->get();
        return $this->sendResult("meals", 200, ['meals' => $meal]);
    }

    public function mobileSmsLeads(Request $request)
    {
        $meal = Lead::select("phone")->groupBy("phone")->get();
        return $this->sendResult($meal->count() . " Leads", 200, ['leads' => $meal]);
    }


    public function couponCheck(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'coupon' => 'required',
            'totalAmount' => 'required',
            'shop_id' => 'required',
        ]);

        if ($validator->fails()) {
            return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), $validator->errors()->first(), $request);
        }

        $data = $request->all();
        $coupon = Coupon::where('code', $data['coupon'])
            ->where("startOn", '<', DateTimeHelper::getLocalNow(1))
            ->where("expiredOn", '>', DateTimeHelper::getLocalNow(1))
            ->whereColumn('timesUsed', '<', 'usesPerCoupon')
            ->where('shop_id', $data['shop_id'])
            ->where('type', 'SHOP')
            ->first();
        $discountAmount = 0;
        if (!empty($coupon)) {
            if ($coupon->discountType == 'AMOUNT') {
                $discountAmount = $coupon->discountValue;
            } else {
                $discountAmount = $data['totalAmount'] / $coupon->discountValue;
                if ($discountAmount > $coupon->discountCap) {
                    $discountAmount = $coupon->discountCap;
                }
            }
            $data = ['discount_amount' => $discountAmount, 'coupon_id' => $coupon->id ?? ''];
            return $this->sendResult("Applied Coupon", 200, $data);
        }
        return $this->sendResult('Invalid coupon', config('codes.REQUEST_VALIDATAION_ERROR'), [], $request);
    }

    public function placeOrder(Request $request)
    {
        try {
            $order = new Order();
            $member = $request->auth;
            $validator = Validator::make($request->all(), [
                'items' => 'required',
                'orderType' => 'required',
                'shop_id' => 'required',
                'totalAmount' => 'required',
            ]);

            if ($validator->fails()) {
                return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), $validator->errors()->first(), $request);
            }

            if ($request->get("orderType") == Order::ORDER_TYPE_TAKEAWAY) {
                $validator = Validator::make($request->all(), [
                    'pickup_time' => 'required'
                ]);
                if ($validator->fails()) {
                    return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), $validator->errors()->first(), $request);
                }
                $order->pickup_time = date("Y-m-d H:i:s", strtotime($request->get('pickup_time')));
            }

            if ($request->get("orderType") == Order::ORDER_TYPE_DELIVERY) {
                $validator = Validator::make($request->all(), [
                    'deliveryAmount' => 'required',
                    'address' => 'required',
                ]);
                if ($validator->fails()) {
                    return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), $validator->errors()->first(), $request);
                }
                $order->deliveryAmount = $request->get("deliveryAmount");
                $order->address = $request->get("address");
            }

            if (!empty($request->get("coupon_id"))) {
                $order->discountAmount = $request->get('discountAmount');
                $order->coupon_id = $request->get('coupon_id');
            }
            DB::beginTransaction();
            $order->member_id = $member->id;
            $order->status = Order::ORDER_STATUS_PROCESS;
            $order->source = 'APP';
            $order->totalAmount = $request->get("totalAmount");
            $order->shop_id = $request->get('shop_id');
            $order->type = $request->get("orderType");

            if (!$order->save())
                throw new \Exception("An Error occurs while creating Order");

            foreach (json_decode($request->get('items')) as $item) {
                $orderItem = new OrderItem();
                $orderItem->create([
                    'order_id' => $order->id,
                    'meal_id' => $item->meal_id,
                    'qty' => $item->qty,
                    'amount' => $item->price
                ]);
                // notification will generates here
            }
            DB::commit();
            return $this->sendResult("Order Confirmed", 200, []);
        } catch (\Exception $exception) {
            DB::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.REQUEST_VALIDATAION_ERROR'), [], $request);
        }
        return $this->sendResult("order not created", config('codes.REQUEST_VALIDATAION_ERROR'), [], $request);
    }

}