<?php


namespace App\Http\Controllers;

use App\Libraries\ResponseCodes;
use App\models\MemberMealServe;
use App\models\MemberMealServeDispositions;
use App\models\MemberReservation;
use App\models\RiderLog;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Validator;
use Illuminate\Http\Request;


class RidersController extends Controller
{
    /*
     * riderOrders*/
    public function parcels(Request $request)
    {
        $query = $this->getMembersReservations($request->get("onDate"))->where("member_reservations.user_id", $request->auth->id)->paginate($request->get("per-page", 10));

        $parcels = $query->toArray();

        return $this->sendResult('parcels fetch successfully.', config('codes.SUCCESS'), ['parcels' => $parcels["data"]]);
    }

    public function getPrintData(Request $request)
    {
        $query = $this->getMembersReservations($request->get("onDate"))->whereNotIn("member_reservations.status",[MemberReservation::STATUS_CANCELED])->get();

        $parcels = $query->toArray();

        return $this->sendResult('parcels fetch successfully.', config('codes.SUCCESS'), ['parcels' => $parcels]);
    }

    private function getMembersReservations($date)
    {
        $members = MemberReservation::join('members', 'members.id', '=', 'member_reservations.member_id')
            ->join('reservations', 'reservations.id', '=', 'member_reservations.reservation_id')
            ->join('meal_plans', 'meal_plans.id', '=', 'reservations.meal_plan_id')
            ->select(
                "member_reservations.id",
                "member_reservations.is_breakfast",
                "member_reservations.is_lunch",
                "member_reservations.is_dinner",
                "member_reservations.is_snack",
                "member_reservations.address",
                "member_reservations.address_note",
                "member_reservations.status",
                "member_reservations.type",
                "meal_plans.title as mealplan",
                "members.name",
                "members.phone",
                "members.email",
                "members.photo",
                "members.id as memberId",
                "reservations.onDate",
                "meal_plans.id as mealplanId")
            ->where("onDate", $date);

        return $members;
    }

    /*
     * riderOrders*/
    public function markDelivered(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required'
        ]);

        if ($validator->fails()) {
            return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), ['errors' => $validator->errors()]);
        }
        try {
            $id = $request->get('id');
            $memberReservation = MemberReservation::where('id', $id)->first();

            if (empty($memberReservation))
                return $this->sendResult('data Not found', config('codes.RESOURCE_NOT_FOUND'));

            if ($memberReservation->status != MemberReservation::STATUS_IN_TRANSIT)
                return $this->sendResult('not In transit to marked Delivered', config('codes.RESOURCE_NOT_FOUND'));

            DB::beginTransaction();

            $memberReservation->status = MemberReservation::STATUS_DELIVERED;

            if (!$memberReservation->save())
                throw new \Exception("change status occurred an error.");

            $memberReservation->addDisposition("Meal parcel as been Delivered", null, null, $request->auth->id);

            DB::commit();
            return $this->sendResult('Parcel successfully Delivered.', config('codes.SUCCESS'));


        } catch (\Exception $exception) {
            Db::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));

        }

    }

    /*
     * riderOrders*/
    public function markDeclined(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'id' => 'required',
            'reason_id' => 'required'
        ]);

        if ($validator->fails()) {
            return $this->sendResult('error in request data', config('codes.REQUEST_VALIDATAION_ERROR'), ['errors' => $validator->errors()]);
        }
        try {
            $id = $request->get('id');
            $memberReservation = MemberReservation::where('id', $id)->first();

            if (empty($memberReservation))
                return $this->sendResult('data Not found', config('codes.RESOURCE_NOT_FOUND'));

            if ($memberReservation->status != MemberReservation::STATUS_IN_TRANSIT)
                return $this->sendResult('not In transit to marked Declined', config('codes.RESOURCE_NOT_FOUND'));

            DB::beginTransaction();

            $memberReservation->status = MemberReservation::STATUS_DECLINED;

            if (!$memberReservation->save())
                throw new \Exception("change status occurred an error.");

            $memberReservation->addDisposition(($request->get("comment", "Meal parcel as been Declined")), $request->get("reason_id"), null, $request->auth->id);

            DB::commit();
            return $this->sendResult('Parcel successfully Declined.', config('codes.SUCCESS'));


        } catch (\Exception $exception) {
            Db::rollBack();
            return $this->sendResult($exception->getMessage(), config('codes.DATA_NOT_SAVE'));

        }

    }


    protected function doServedStatus(MemberMealServe $model): void
    {
        $model->status = MemberMealServe::STATUS_SERVED;;
        if (!$model->save())
            throw new \Exception("An error occur while updating reserves status");

        $memberMealServeDispoModel = new MemberMealServeDispositions([
            "member_meal_serve_id" => $model->id,
            "status" => $model->status,
            "comment" => "This meal is now {$model->status}",
            "user_id" => $this->_user->id,
        ]);

        if (!$memberMealServeDispoModel->save())
            throw new \Exception("An error occur while saving status Disposition");
    }

    public function markInTransit(Request $request)
    {
        if (!$request->has("ids")) {
            $this->_response['code'] = ResponseCodes::DATA_NOT_FOUND;
            $this->_response['message'] = "No id match";
        }

        $ids = $request->get('ids');

        $memberMealServes = MemberMealServe::whereIn('id', explode(",", $ids))->get();

        if (empty($memberMealServes)) {
            $this->_response['code'] = ResponseCodes::DATA_NOT_FOUND;
            $this->_response['message'] = "no order found";
        }

        try {
            DB::beginTransaction();
            foreach ($memberMealServes as $memberMealServe) {
                if (empty($memberMealServe) || $memberMealServe->status != MemberMealServe::STATUS_COOKING) {
                    continue;
                }
                $this->doTransitStatus($memberMealServe);
            }
            DB::commit();

        } catch (\Exception $exception) {
            Db::rollBack();
            $this->_response['code'] = $exception->getCode();
            $this->_response['message'] = $exception->getMessage();
        }

        return $this->response();
    }

    public function log(Request $request)
    {
        try {
            DB::beginTransaction();
            foreach (json_decode($request->get("rows")) as $row) {
                $this->createLog($row->latitude, $row->longitude);
            }
            DB::commit();
        } catch (\Exception $exception) {
            Db::rollBack();
            $this->_response['code'] = $exception->getCode();
            $this->_response['message'] = $exception->getMessage();
        }
        return $this->response();
    }

    protected
    function doTransitStatus(MemberMealServe $model)
    {
        $model->user_id = $this->_user->id;
        $model->status = MemberMealServe::STATUS_IN_TRANSIT;

        if (!$model->save())
            throw new \Exception("An error occur while updating reserves status");

        $memberMealServeDispoModel = new MemberMealServeDispositions([
            "member_meal_serve_id" => $model->id,
            "status" => $model->status,
            "comment" => "This meal is in transit for Transport",
            "user_id" => $this->_user->id,
        ]);

        if (!$memberMealServeDispoModel->save())
            throw new \Exception("An error occur while saving status Disposition");

        return $model;
    }

    protected function createLog($lat, $long)
    {
        $model = new RiderLog([
            "rider_id" => $this->_user->id,
            "longitude" => $long,
            "latitude" => $lat
        ]);

        if (!$model->save())
            throw new \Exception("An error occur while saving log");

        return $model;
    }


}
