<?php

namespace App\Http\Middleware;


use App\Http\Controllers\Controller;
use App\models\Member;
use App\User;
use Closure;
use Firebase\JWT\ExpiredException;
use Firebase\JWT\JWT;
use Illuminate\Http\Request;

class JwtMiddleware extends Controller
{


    public function handle(Request $request, Closure $next, $guard = null)
    {

        $token = $request->header('token');

        if (!$token) {
            // Unauthorized response if token not there
            return $this->sendResult('Token not provided', config('codes.TOKENMISSING'), null, $request);
        }

        try {
            $credentials = JWT::decode($token, env('JWT_SECRET'), ['HS256']);
        } catch (ExpiredException $e) {
            return $this->sendResult('Provided token is expired.', config('codes.TOKENEXPIRE'), null, $request);
        } catch (\Exception $e) {
            return $this->sendResult('An error while decoding token.', config('codes.INVALIDTOKEN'), null, $request);
        }
        if (request()->is('members*')) {
            if (empty($credentials->user->phone)) {
                return $this->sendResult('Customer Phone Number Not Found', config('codes.USERNOTFOUND'), null, $request);
            }
            $customer = Member::where('phone', $credentials->user->phone)->first();
            if (empty($customer)) {
                return $this->sendResult('Customer not found', config('codes.USERNOTFOUND'), null, $request);
            }
            $request->auth = $customer;

        } elseif (request()->is('users*')) {

            $user = User::where('email', $credentials->user->email)->first();

            if (empty($user)) {
                return $this->sendResult('Vendor not found', config('codes.USERNOTFOUND'), null, $request);
            }
            // Now let's put the user in the request class so that you can grab it from there
            $request->auth = $user;
        } else {
            return $this->sendResult('Member/User not found', config('codes.USERNOTFOUND'), null, $request);
        }

        return $next($request);
    }
}
