<?php

namespace App\Http\Middleware;

use App\Libraries\ResponseCodes;
use App\models\Users;
use Closure;
use Firebase\JWT\ExpiredException;
use Firebase\JWT\JWT;

class JwtUserMiddleware
{
    /**
     * Handle an incoming request.
     *
     * @param \Illuminate\Http\Request $request
     * @param \Closure $next
     * @return mixed
     */
    public function handle($request, Closure $next, $guard = null)
    {
        $token = $request->header('token');
        if (!$token) {
            // Unauthorized response if token not there
            return response()->json([
                'code' => ResponseCodes::TOKENMISSING,
                'message' => "Token is not provided",
                'data' => (object)[]
            ], 200);
        }
        try {
            $credentials = JWT::decode($token, env('JWT_SECRET'), ['HS256']);

        } catch (ExpiredException $e) {
            return response()->json([
                'code' => ResponseCodes::INVALIDTOKEN,
                'message' => "Provided token is expired.",
                'data' => (object)[],
            ], 200);

        } catch (\Exception $e) {

            return response()->json([
                'code' => ResponseCodes::INVALIDTOKEN,
                'message' => "An error while decoding token.",
                'data' => (object)[]
            ], 200);
        }

        $currentUser = Users::find($credentials->sub->id);
        if (empty($currentUser)) {
            return response()->json([
                'code' => ResponseCodes::USERNOTFOUND,
                'message' => "This user in not found.",
                'data' => (object)[]
            ], 200);
        }
        $currentUser = $currentUser->getAttributes();

        $request->merge(compact('currentUser'));
        return $next($request);
    }
}
