<?php


namespace App\managers;


use App\models\FcmMessage;
use App\models\FcmResponse;
use App\models\Member;
use Illuminate\Support\Facades\Auth;

class FcmManager
{


    public static function sendAutoNotificationByStatus(Member $member, $data = [])
    {
        if(env("APP_ENV") == "local")
            return false;

        if (empty($member) || empty($member->fcmAppToken) || empty($member->fcmAppToken->fcm_token))
            return false;

        $data["fcm_token"] = $member->fcmAppToken->fcm_token;
        $data["member_id"] = $member->id;
        $data["domain_url"] = (!empty($data["domain_url"])) ? $data["domain_url"] : "https://www.dietbox.pk";
        $data["image"] = (!empty($data["image"])) ? $data["image"] : "https://dietbox.pk/aron-assets/images/logo.svg";

        $response = SELF::send($data);

        return true;
    }


    public static function sendByMemberIds(FcmMessage $messageModel, Array $memberIds)
    {
        if (empty($messageModel))
            return false;

        foreach ($memberIds as $memberId) {
            $member = Member::findOrFail($memberId);
            if (!empty($member) && !empty($member->fcmAppToken)) {
                $response = SELF::send([
                    "fcm_token" => $member->fcmAppToken->fcm_token,
                    "title" => $messageModel->title,
                    "image" => ($messageModel->image)?env("IMAGE_URL")."/".$messageModel->image:"https://dietbox.pk/aron-assets/images/logo.svg",
                    "message" => $messageModel->message,
                    "domain_url" => $messageModel->domain_url,
                    "fcm_message_id" => $messageModel->id,
                    "member_id" => $member->id
                ]);
                if (!$response)
                    throw new \Exception("something went wrong ....");
            }

        }
    }

    public static function send(Array $data)
    {
        // data ...
        $to = !empty($data['fcm_token']) ? $data['fcm_token'] : "notfound";
        $title = $data['title'];
        $image_url = $data['image'];
        $message = $data['message'];
        $action_url = $data['domain_url'];
        $api_key = "1:923413281002:android:57db448895347e4790bf93";
        $server_key = "AAAA1v-4eOo:APA91bG8-eMmKWmGgWqwawpVS5lyxn458-w2Df1EjK_A_OgkwzfYaWI5sg6B682sJ1fB9EBgI___yXNmsmTm9q8XeA63Bf4eftBx6O0NfKNUHJY_LjtTATso1Cd6f-Wxgr42IwSMmaSV";

        $msg = array(
            'body' => $message,
            'title' => $title,
            'click_action' => $action_url,
            'icon' => $image_url,/*Default Icon*/
            'sound' => 'mySound'/*Default sound*/
        );
        $fields = array(
            'to' => $to,
            'notification' => $msg
        );

        $headers = array
        (
            'Authorization: key=' . $server_key,
            'Content-Type: application/json',
            "cache-control: no-cache"
        );

        $curl = curl_init();
        curl_setopt($curl, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
        curl_setopt($curl, CURLOPT_POST, true);
        curl_setopt($curl, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($fields));
        $response = curl_exec($curl);
        $err = curl_error($curl);
        curl_close($curl);

        if ($err)
            throw new \Exception($err);

        $resdecoded = json_decode($response, true);
        $resp = (string)$response;

        $fcm_reponse = new FcmResponse([
            "user_token" => $data['fcm_token'],
            "fcm_message_id" => (!empty($data['fcm_message_id'])) ? $data['fcm_message_id'] : null,
            "multicast_id" => (!empty($resdecoded['multicast_id'])) ? $resdecoded['multicast_id'] : '',
            "status" => $resdecoded['success'],
            "fcm_response" => $resp,
            "member_id" => $data['member_id'],
        ]);

        if (Auth::user())
            $fcm_reponse->user_id = Auth::user()->id;

        $fcm_reponse->title = $title;
        $fcm_reponse->message = $message;
        $fcm_reponse->image = $image_url;

        $fcm_reponse->save();
        return true;
    }


}
