<?php

namespace App\models;

use App\Libraries\ModelDependentUpload;
use App\TraitLibraries\ModelHelper;
use App\TraitLibraries\ModelSearch;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class FcmMessage extends Model
{
    use ModelSearch, ModelHelper, ModelDependentUpload;

    protected $fillable = [
        "title",
        "message",
        "image",
        "domain_url",
        "status",
    ];

    public $rule = array(
        'title' => 'required|min:3',
        'message' => 'required|min:3',
    );

    /**
     * The search attributes that are mass assignable.
     *
     * @var array
     */

    public $searchableColumns = array(
        'title' =>
            array(
                'dataType' => 'like',
                'join' => 'fcm_responses.title'
            ),
        'created' =>
            array(
                'dataType' => 'daterange',
                'join' => 'fcm_responses.created_at'
            ),
    );

    public function scopeFcmHistory($query)
    {

        $query->leftJoin('fcm_responses', 'fcm_responses.fcm_message_id', '=', 'fcm_messages.id');
        $query->leftJoin('members', 'members.id', '=', 'fcm_responses.member_id');

        $query->select('fcm_messages.*',
            'fcm_messages.title',
            'fcm_messages.message',
            'fcm_messages.image',
            'members.name',
            'fcm_responses.member_id'
        );

        return $query;
    }
}
