<?php

namespace App\models;

use App\TraitLibraries\ModelHelper;
use App\TraitLibraries\ModelSearch;
use App\User;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

/**
 * @property mixed discountPercent
 * @property mixed discount_coupon_id
 * @property false|string parameters
 * @property mixed object
 * @property mixed objectId
 * @property string title
 * @property int created_by
 * @property mixed member_id
 */
class Invoice extends Model
{
    use ModelHelper, ModelSearch;

    protected $fillable = [
        'title',
        'description',
        'member_id',
        'lead_id',
        'object',
        'objectId',
        'cancel_by',
        'created_by',
        'status',
        'type',
        'cancelDate',
        'paidDate',
        'isPaid',
        'isRefund',
        'discount_coupon_id',
        'discountPercent',
        'parameters'
    ];

    //Types ...
    public const TypeSale = "SALE";

    //statuses ...
    public const PENDING = "PENDING";
    public const VOID = "VOID";
    public const PARTIAL = "PARTIAL";
    public const CANCEL = "CANCEL";

    //Object ...
    public const ObjectMealplan = "MEAL_PLANS";
    public const ObjectIngredients = "INGREDIENTS";
    public const ObjectExpense = "EXPENSE";
    public const ObjectOrders = "ORDERS";


    public function invoiceItems()
    {
        return $this->hasMany(InvoiceItem::class);
    }

    public function getInvoiceTotalAttribute()
    {
        $total = 0;

        foreach ($this->invoiceItems as $item):
            $total += $item->amount;
        endforeach;

        return $total;
    }


    public function getPaidTotalAttribute()
    {
        $total = 0;

        foreach ($this->paidTransactions as $item):
            $total += $item->amount;
        endforeach;

        return $total;
    }

    public function transactions()
    {
        return $this->hasMany(Transaction::class);
    }

    public function paidTransactions()
    {
        return $this->hasMany(Transaction::class)->where('status', '=', Transaction::SUCCESS)
            ->where('type', '=', Transaction::TYPE_CHARGES);
    }

    public function paidRefundTransactions()
    {
        return $this->hasMany(Transaction::class)->where('status', '=', Transaction::SUCCESS)
            ->where('type', '=', Transaction::TYPE_REFUND);
    }

    public function pendingTransactions()
    {
        return $this->hasMany(Transaction::class)->where('status', '=', Transaction::PENDING);
    }


    public function mealplan()
    {
        return $this->belongsTo(MealPlan::class, 'objectId');
    }

    public function member()
    {
        return $this->belongsTo(Member::class);
    }


}
