<?php

namespace App\models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

/**
 * @property int totalGrams
 * @property int totalCalories
 * @property mixed rating
 */
class Meal extends Model
{

    protected $fillable = [
        'title',
        'description',
        'forShop',
        'type',
        'perOrderAmount',
        'logo',
        'rating',
        'totalCalories',
        'totalGrams',
        'cover_id'
    ];

    const BREAKFAST = "BREAKFAST";
    const DINNER = "DINNER";
    const LUNCH = "LUNCH";
    const SNACKS = "SNACKS";


    public function getRecipesAttribute()
    {
        $imagePath = env("IMAGE_BASE_URL");
        $query = new Recipe();
        $data = $query->join("meal_recipes", "recipes.id", "=", "meal_recipes.recipe_id")
            ->leftJoin("files", "files.id", "=", "recipes.cover_id")
            ->where("meal_recipes.meal_id", $this->id)
            ->select("recipes.id",
                "recipes.title",
                "recipes.cuisines",
                "recipes.description",
                "recipes.type",
                "recipes.rating",
                "recipes.totalCalories",
                "recipes.totalGrams",
                "recipes.cover_id",
                "recipes.active",
                DB::raw("CONCAT('$imagePath/',files.filePath,'/',files.title) as logo"))->get();

        return $data;
    }

    public function getNutritionAttribute()
    {
        return $this->hasMany(MealRecipe::class, "meal_id")
            ->join('recipe_ingredients', 'meal_recipes.recipe_id', '=', 'recipe_ingredients.recipe_id')
            ->join('ingredient_nutrition', 'recipe_ingredients.ingredient_id', '=', 'ingredient_nutrition.ingredient_id')
            ->join('nutrition', 'nutrition.id', '=', 'ingredient_nutrition.nutrition_id')
            ->select('nutrition.title', 'nutrition.unit', DB::raw('SUM( recipe_ingredients.grams * ingredient_nutrition.value) as `value`'))
            ->groupBy('nutrition.id')->get();

    }


}
