<?php

namespace App\models;

use App\TraitLibraries\ModelHelper;
use Carbon\Carbon;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Illuminate\Contracts\Auth\MustVerifyEmail;


class Member extends Authenticatable implements MustVerifyEmail
{

    use ModelHelper;
    protected $fillable = [
        'id',
        'name',
        'email',
        'phone',
        'photo',
        'weight',
        'height',
        'gender',
        'bdate',
        'reference',
        'mobile_verified_at',
        'state',
        'address',
        'city',
    ];

    public function save(array $options = [])
    {

        $saved = parent::save($options); // TODO: Change the autogenerated stub

        if ($this->wasChanged(['weight'])) {
            MemberScale::create(['member_id' => $this->id, "value" => $this->weight]);
        }

        return $saved;
    }

    //v3 ..
    public function updateFcmToken($fcmToken)
    {
        $fcmAppTokenModel = FcmAppToken::where('member_id', '=', $this->id)->where("status", '=', 'Y')->first();

        if (!empty($fcmAppTokenModel)) {
            $fcmAppTokenModel->status = 'N';
            if (!$fcmAppTokenModel->save()) {
                throw new \Exception("An error occurred on Saving Fcm");
            }
        }

        $fcmAppTokenModel = new FcmAppToken([
            "member_id" => $this->id,
            "status" => 'Y',
        ]);

        $fcmAppTokenModel->fcm_token = $fcmToken;

        if (!$fcmAppTokenModel->save()) {
            throw new \Exception("An error occurred on Saving Fcm");
        }
    }

    //v3 ..
    public function updateWeight()
    {
        if (!$this->isDirty("weight")) {
            $primaryAddress = new MemberScale([
                "value" => $this->weight,
                "member_id" => $this->id
            ]);
            $primaryAddress->save();
        }
    }

    //v3 ..
    public function addPrimaryAddress($address, $address_note = null)
    {
        $primaryAddress = MemberAddress::where('member_id', '=', $this->id)->where("type", MemberAddress::TYPE_PRIMARY)->first();

        if (!$primaryAddress)
            $primaryAddress = new MemberAddress([
                "type" => MemberAddress::TYPE_PRIMARY,
                "member_id" => $this->id,
            ]);

        $primaryAddress->address = $address;

        if ($address_note)
            $primaryAddress->address_note = $address_note;

        $primaryAddress->save();
    }

    //v3 ..
    public function addSecondaryAddress($address = null, $data = [])
    {
        $secondaryAddress = MemberAddress::where('member_id', '=', $this->id)->where("type", MemberAddress::TYPE_SECONDARY)->first();

        if (!$secondaryAddress)
            $secondaryAddress = new MemberAddress([
                "type" => MemberAddress::TYPE_SECONDARY,
                "member_id" => $this->id
            ]);


        if ($address)
            $secondaryAddress->address = $address;

        $secondaryAddress->address_note = $data['address_note'];

        $secondaryAddress->mon = (!empty($data['mon'])) ?? 0;
        $secondaryAddress->tue = (!empty($data['tue'])) ?? 0;
        $secondaryAddress->wed = (!empty($data['wed'])) ?? 0;
        $secondaryAddress->thu = (!empty($data['thu'])) ?? 0;
        $secondaryAddress->fri = (!empty($data['fri'])) ?? 0;
        $secondaryAddress->sat = (!empty($data['sat'])) ?? 0;
        $secondaryAddress->sun = (!empty($data['sun'])) ?? 0;

        $secondaryAddress->save();
    }

    public function activeMealplan()
    {
        $query = $this->hasOne(MemberMealPlan::class)->orderBy("expireOn", "DESC");
        $query->with('mealplan');
        return $query;
    }

    //v3
    public function isValidMealplanOfMember($meal_plan_id, $date)
    {
        $memberMealPlan = Member::join("member_meal_plans", "members.id", "=", "member_meal_plans.member_id")
            ->join("meal_plans", "meal_plans.id", "=", "member_meal_plans.meal_plan_id")
            ->where("members.id", $this->id)
            ->where("meal_plans.id", $meal_plan_id)
            ->where("member_meal_plans.expireOn", ">=", $date)
            ->where("member_meal_plans.startOn", "<=", $date)
            ->select("member_meal_plans.*")->first();

        return !empty($memberMealPlan) ? $memberMealPlan->id : 0;
    }

    public function mealplans()
    {
        $query = $this->hasMany(MemberMealPlan::class)->orderBy("expireOn", "DESC");
        $query->with('mealplan');
        return $query;
    }

    public function addresses()
    {
        return $this->hasMany(MemberAddress::class);
    }

    public function invoices()
    {
        return $this->hasMany(Invoice::class)->orderBy("id", "desc")->with("invoiceItems", "paidTransactions");
    }


    public function memberScale()
    {
        return $this->hasOne(MemberScale::class)->orderBy("id", "DESC");
    }


    public function primaryAddress()
    {

        return $this->hasOne(MemberAddress::class)->where("type", MemberAddress::TYPE_PRIMARY);
    }

    public function secondaryAddress()
    {
        return $this->hasOne(MemberAddress::class, "member_id")->where("type", MemberAddress::TYPE_SECONDARY);
    }

    public function activities()
    {
        return $this->belongsToMany(Activity::class, 'member_activities', 'member_id', 'activity_id');
    }

    public function getPrimaryAddress()
    {

        $primaryAddress = $this->primaryAddress;

        return $primaryAddress->address;
    }

    public function getDeliveryAddress($date)
    {
        $secondaryAddress = $this->secondaryAddress;
        $address = "No address please call member";
        if ($secondaryAddress && $secondaryAddress->address) {
            $address = $secondaryAddress->address;
            $day = Carbon::parse($date)->format("D");
            $dayValue = $secondaryAddress->getAttribute(strtolower($day));
            if ($dayValue) {
                return $secondaryAddress;
            }
        }

        $primaryAddress = $this->primaryAddress;

        if (!$primaryAddress->address)
            $primaryAddress->setAttribute("address", $address);

        return $primaryAddress;
    }


}
