<?php

namespace App\models;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Recipe extends Model
{
    protected $fillable = [
        'title',
        'logo',
        'cuisines',
        'description',
        'created_by'
    ];

    protected $appends = ["nutrition"];

    public function ingredients()
    {
        return $this->belongsToMany(Ingredient::class, 'recipe_ingredients', 'recipe_id', 'ingredient_id')->withPivot("grams");
    }


    public function getNutritionAttribute()
    {
        $recipeIngredients = new RecipeIngredients();
        $query = $recipeIngredients->newQuery();
        $query
            ->join('ingredients', 'recipe_ingredients.ingredient_id', '=', 'ingredients.id')
            ->join('ingredient_nutrition', 'ingredient_nutrition.ingredient_id', '=', 'ingredients.id')
            ->join('nutrition', 'ingredient_nutrition.nutrition_id', '=', 'nutrition.id')
            ->where('recipe_ingredients.recipe_id', $this->id);
        $query->select('nutrition.title', 'nutrition.unit',
            DB::raw('SUM(`ingredient_nutrition`.`value` * `recipe_ingredients`.`grams`) as `value`'));

        $query->groupBy('ingredient_nutrition.nutrition_id');

        return $query->get();
    }

}
