<?php

namespace App\models;

use App\TraitLibraries\ModelHelper;
use App\TraitLibraries\ModelSearch;
use Illuminate\Database\Eloquent\Model;

/**
 * @property mixed snack_id
 * @property mixed breakfast_id
 * @property mixed dinner_id
 * @property mixed lunch_id
 */
class Reservation extends Model
{
    use ModelHelper, ModelSearch;

    protected $fillable = array(
        "breakfast_id",
        "lunch_id",
        "snack_id",
        "dinner_id",
        "meal_plan_id",
        "status",
        "onDate",
        "user_id",
        "onCookingCount",
        "inTransitCount",
        "deliveredCount",
        "canceledCount",
        "declinedCount"
    );

    const STATUS_IN_PROCESS = "IN_PROCESS";
    const STATUS_PARTIAL_SERVED = "PARTIAL_SERVED";
    const STATUS_ALL_SERVED = "ALL_SERVED";
    const STATUS_PENDING = "PENDING";
    const STATUS_CANCELED = "CANCELED";


    public function mealplan()
    {
        return $this->belongsTo(MealPlan::class, 'meal_plan_id');
    }

    public function breakfast()
    {
        return $this->belongsTo(Meal::class, 'breakfast_id');
    }

    public function dinner()
    {
        return $this->belongsTo(Meal::class, 'dinner_id');
    }

    public function lunch()
    {
        return $this->belongsTo(Meal::class, 'lunch_id');
    }

    public function snack()
    {
        return $this->belongsTo(Meal::class, 'snack_id');
    }


    /**
     * @param $query
     * @return mixed
     */
    public function scopePackageMenu($query)
    {
        //reservations plan
        $query->leftJoin('meals as breakfastMeal', 'breakfastMeal.id', '=', 'reservations.breakfast_id');
        $query->leftJoin('meals as lunchMeal', 'lunchMeal.id', '=', 'reservations.lunch_id');
        $query->leftJoin('meals as snackMeal', 'snackMeal.id', '=', 'reservations.snack_id');
        $query->leftJoin('meals as dinnerMeal', 'dinnerMeal.id', '=', 'reservations.dinner_id');

        //select columns ...
        $query->select(
            'reservations.*',
            'breakfastMeal.title as breakfast',
            'lunchMeal.title as lunch',
            'snackMeal.title as snack',
            'dinnerMeal.title as dinner',
            'breakfastMeal.totalCalories as breakfastCalories',
            'lunchMeal.totalCalories as lunchCalories',
            'breakfastMeal.totalGrams as breakfastGrams',
            'lunchMeal.totalGrams as lunchGrams',
            'snackMeal.totalCalories as snackCalories',
            'snackMeal.totalGrams as snackGrams',
            'dinnerMeal.totalCalories as dinnerCalories',
            'dinnerMeal.totalGrams as dinnerGrams'
        );

        return $query;
    }

}
